﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ExportCegidParam.BO;

namespace ExportCegidParam.BLL
{
    public class MgtExport
    {
        #region Attribut
        private static MgtExport _instance = null; // instance de la classe
        private static SQL.MSSQL _connexion = null;  // instance de la classe qui sert à la connexion
        private static List<string> _arguments = null; // liste de chaine qui contient les parametres passé a l'exécutable
        private static List<Parametrage> _listParamatrages = null; //Récupération des données de CEGID, contient tous les paramétrage du client
        private static DataExport _listData = null; //données à exporter, la requête est contenue dans un fichier passé en paramatres de l'objet Parametrage
        private static int _comptParametrage = 0; // Compteur de paramétrage
        #endregion

        #region Singleton
        private MgtExport(string[] args)
        {
            if (this.CheckVariableApp()){
                _arguments = new List<string>();
                foreach (string item in args) { _arguments.Add(item); }
                _connexion = new SQL.MSSQL(_arguments);
                FILE.Log.WriteLine(_connexion._strConnexion);
                if (!_connexion.TesterConnexionBDDSurTiers()) { throw new ApplicationException("Erreur la base de donnée est inacéssible"); }
            }
            else{
                throw new ApplicationException("Erreur toutes les variables liées au bon fonctionnement de l'application ne sont pas initialisées");
            }
        }
        public static MgtExport getInstance(string[] args = null)
        {
            if (MgtExport._instance == null || _connexion == null)
            {
                MgtExport._instance = new MgtExport(args);
            }
            return MgtExport._instance;
        }
        #endregion

        #region Accesseur
        public List<string> Arguments
        {
            get { return _arguments; }
        }
        public int ParametrageCompteur
        {
            get { return _comptParametrage; }
        }
        public int ParametrageTotal
        {
            get { return _listParamatrages.Count; }
        }
        #endregion

        #region Methode Public
        /// <summary>
        /// méthode d'export, à lancé après l'import, ici on fera les tests pour savoir si un export doit être effectuer 
        /// et rediriger vers la bonne sérialisation
        /// </summary>
        public void ExportVersFichier()
        {
            _listParamatrages = SQL.SQLParametrage.GetParametrages(_connexion);
            foreach (Parametrage parametrage in _listParamatrages)
            {
                if (parametrage.IsValid){
                    try
                    {
                        string pathfile = System.IO.Path.Combine(parametrage.PathDirDest, parametrage.NameFile);
                        Console.WriteLine("\nDébut de l'export " + parametrage.Libelle + ", génération du fichier " + pathfile);
                        FILE.Log.WriteLine("Début de l'export " + parametrage.Libelle + ", génération du fichier " + pathfile);
                        string requete = FILE.INI.ReadRequete(parametrage.PathFileSQL);
                        FILE.Log.WriteLine("Début de l'export " + parametrage.Libelle + ", génération du fichier " + pathfile);
                        _listData = SQL.SQLDataExporter.GetDataExport(_connexion, requete);
                        if (_listData.Lignes.Count < 1){
                            Console.WriteLine("Aucune données à exporter");
                            FILE.Log.WriteLine("Aucune données à exporter, export : " + parametrage.Libelle + ", fichier SQL : " + parametrage.PathFileSQL);
                        } else {
                            Serialisation.Serialiseur.getSerializer(pathfile, _listData, parametrage);
                            if (!System.IO.File.Exists(pathfile)){
                                Console.WriteLine("Erreur le fichier" + pathfile + "n'a pas été généré");
                                FILE.Log.WriteLine("Erreur le fichier" + pathfile + "n'a pas été généré");
                            } else {
                                _comptParametrage += 1;
                                SQL.SQLParametrage.UpdateDateExport(_connexion,parametrage);
                                if (parametrage.IsSendFTP){
                                    Console.WriteLine("Envoie du fixhier " + pathfile + " sur le serveur FTP grâce à la session : " + parametrage.SessionWinSCP);
                                    SendWindSCP(parametrage);
                                }
                                ArchiveFileExport(pathfile, parametrage.NameFile, parametrage.IsSendFTP);
                            }
                        
                        }
                    }
                    catch (Exception ex)
                    {
                        Console.WriteLine(ex.Message);
                        FILE.Log.WriteLine(ex.Message);
                    }
                }
            }
        }
        #endregion

        #region Methode Prive
        /// <summary>
        /// Méthode privé de classe qui va nous dire si tous les éléments sont renseingnés pour l'éxécution du programme
        /// </summary>
        /// <returns></returns>
        private bool CheckVariableApp()
        {
            if ((Properties.Settings.Default.SRV != null || Properties.Settings.Default.SRV != "") && 
                (Properties.Settings.Default.BDD != null || Properties.Settings.Default.BDD != "") &&
                (Properties.Settings.Default.USR != null || Properties.Settings.Default.USR != "") &&
                (Properties.Settings.Default.PWD != null || Properties.Settings.Default.PWD != "") &&
                (Properties.Settings.Default.CLIENT != null || Properties.Settings.Default.CLIENT != "")){
                return true;
            }else{
                return false;
            }
        }
        /// <summary>
        /// Va saugegarder le fichier export généré dans un dossier archive
        /// </summary>
        /// <param name="pathfile"></param>
        private void ArchiveFileExport(string pathfile, string namefile, bool sendFTP)
        {
            string dirParent = System.IO.Directory.GetParent(pathfile).FullName;
            string dirArchive = System.IO.Path.Combine(dirParent, DateTime.Today.ToString("yyyyMMdd"));
            if (!System.IO.Directory.Exists(dirArchive)){
                System.IO.Directory.CreateDirectory(dirArchive);
            }
            if (sendFTP){
                System.IO.File.Move(pathfile, System.IO.Path.Combine(dirArchive, DateTime.Now.ToString("yyyyMMdd_HHmmss_") + namefile));
            }else{
                System.IO.File.Copy(pathfile, System.IO.Path.Combine(dirArchive, DateTime.Now.ToString("yyyyMMdd_HHmmss_") + namefile));
            }
        }
        /// <summary>
        /// envoi le fichier généré sur un serveur ftp définie dans le paramétrage
        /// </summary>
        /// <param name="parametrage"></param>
        private void SendWindSCP(Parametrage parametrage)
        {
            try
            {
                //création du fichier test de script
                string pathFileScript = System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "EXPORT_FTP.txt");
                if (System.IO.File.Exists(pathFileScript)) { System.IO.File.Delete(pathFileScript); }
                using (System.IO.StreamWriter sw = new System.IO.StreamWriter(pathFileScript, false, Encoding.ASCII))
                {
                    sw.WriteLine(@"open """ + parametrage.SessionWinSCP + "");
                    sw.WriteLine("option batch on");
                    sw.WriteLine("option confirm off");
                    sw.WriteLine("cd /");
                    sw.WriteLine(@"lcd """ + parametrage.PathDirDest + "");
                    sw.WriteLine("put " + parametrage.NameFile);
                    sw.WriteLine("exit");
                    sw.Flush();
                    sw.Close();
                }
                //transfere via WINSCP
                Outils.Process.Execute(parametrage.PathWinSCP, @"/console /script=""" + pathFileScript + "");
                //SUppression du fichier de script
                if (System.IO.File.Exists(pathFileScript)) { System.IO.File.Delete(pathFileScript); }
            }
            catch (Exception ex)
            {
                throw new ApplicationException("Erreur lors de l'envoi par FTP, fichier :" + parametrage.PathWinSCP + 
                    ", session : " + parametrage.SessionWinSCP + 
                    ", Erreur : " + ex.Message);
            }
        }
        #endregion


        

    }
}
