﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ExportCegidParam.BO
{
    public class Parametrage
    {
        #region Attribut
        private int _code;
        private String _libelle;
        private String _pathFileSQL;
        private String _pathDirDest;
        private TypeExtention _extention;
        private Encoding _encodage;
        private bool _entete;
        private char _separateur;
        private String _nameFile;
        private TypeFormatNameFile _format;
        private TypeFrequence _frequence;
        private DateTime _dateDernExport;
        private int _numeroJourExport;
        private String _pathWinSCP;
        private String _sessionWinSCP;
        #endregion
  
        #region Propriete
        public int Code
        {
            get { return _code; }
            set { _code = value; }
        }
        public String Libelle
        {
            get { return _libelle; }
            set { _libelle = value; }
        }
        public String PathFileSQL
        {
            get { return _pathFileSQL; }
            set { _pathFileSQL = value; }
        }
        public String PathDirDest
        {
            get { return _pathDirDest; }
            set { _pathDirDest = value; }
        }
        public TypeExtention Extention
        {
            get
            {   
                if (_extention.Equals(null)) { _extention = TypeExtention.csv; }
                return _extention;
            }
            set { _extention = value; }
        }
        public Encoding Encodage
        {
            get
            {
                if (_encodage == null) { _encodage = Encoding.Default; }
                return _encodage;
            }
            set { _encodage = value; }
        }
        public bool Entete
        {
            get { return _entete; }
            set { _entete = value; }
        }
        public char Separateur
        {
            get
            {
                if (Char.MinValue.Equals(_separateur)) { _separateur = ';'; }
                return _separateur;
            }
            set { _separateur = value; }
        }
        public String NameFile
        {
            get
            {
                string nameFile = "";
                switch (_format)
                {
                    case TypeFormatNameFile.Nothing:
                        nameFile = _nameFile + "." + _extention.ToString().ToLower();
                        break;
                    case TypeFormatNameFile.WithDate:
                        nameFile = DateTime.Today.ToString("yyyyMMdd") + "_" + _nameFile + "." + _extention.ToString().ToLower();
                        break;
                    case TypeFormatNameFile.WithSociete:
                        nameFile = _nameFile + "." + _extention.ToString().ToLower();
                        break;
                    case TypeFormatNameFile.WithDateSociete:
                        nameFile = _nameFile + "." + _extention.ToString().ToLower();
                        break;
                    default:
                        break;
                }
                return nameFile;
            }
            set { _nameFile = value; }
        }
        public TypeFormatNameFile Format
        {
            get
            {   if (_format.Equals(null)) { _format = TypeFormatNameFile.Nothing; }
                return _format;
            }
            set { _format = value; }
        }
        public TypeFrequence Frequence
        {
            get
            {
                if (_frequence.Equals(null)) { _frequence = TypeFrequence.Ponctuel;}
                _frequence = TypeFrequence.Ponctuel;
                return _frequence;
            }
            set { _frequence = value; }
        }
        public DateTime DateDernExport
        {
            get { return _dateDernExport; }
            set { _dateDernExport = value; }
        }
        public int NumeroJourExport
        {
            get { return _numeroJourExport; }
            set { _numeroJourExport = value; }
        }
        public String PathWinSCP
        {
            get
            {
                return _pathWinSCP;
            }
            set { _pathWinSCP = value; }
        }
        public String SessionWinSCP
        {
            get { return _sessionWinSCP; }
            set { _sessionWinSCP = value; }
        }
        #endregion

        #region Propriété calculé
        public bool IsValid
        {
            get
            {
                //vérifie la non nullité des éléments indispensables
                if (this.NameFile == null || this.NameFile.Equals("")){
                    return false;
                }
                else if (this.PathFileSQL == null || !(System.IO.File.Exists(this.PathFileSQL))){
                    return false;
                }
                else if (this.PathDirDest == null || !(System.IO.Directory.Exists(this.PathDirDest))){
                    return false;
                }
                else if ((this.Extention.Equals(TypeExtention.csv) || this.Extention.Equals(TypeExtention.txt)) && this.Separateur.Equals(null)){
                    return false;
                }
                else if ((this.Frequence.Equals(TypeFrequence.Hebdomadaire) && (this.NumeroJourExport < 1 || this.NumeroJourExport < 7) ||
                (this.Frequence.Equals(TypeFrequence.Mensuel) && (this.NumeroJourExport < 1 || this.NumeroJourExport < 30)))){
                    return false;
                }
                //Vérifie si l'export doit s'effectué par rapport à la date et l&a fréquence enregistré
                if ((this.Frequence.Equals(TypeFrequence.Quotidient)) && !(this.DateDernExport.Date < DateTime.Now.Date)){
                    return false;
                }
                else if ((this.Frequence.Equals(TypeFrequence.Hebdomadaire)) && !(this.DateDernExport.Date < DateTime.Now.Date) && !(this.NumeroJourExport.Equals(((((Int32)(DateTime.Today.DayOfWeek + 6)) % 7) + 1)))){
                    return false;
                }
                else if ((this.Frequence.Equals(TypeFrequence.Mensuel)) && !(this.DateDernExport.Date < DateTime.Now.Date) && !(this.NumeroJourExport.Equals(DateTime.Today.Day))){
                    return false;
                }
                return true;
            }
        }
        public bool IsSendFTP
        {
            get
            {
                if (!(this.SessionWinSCP.Equals(null) || this.SessionWinSCP.Equals(""))){
                    if (!System.IO.File.Exists(this.PathWinSCP)){
                        if (System.IO.File.Exists(System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "WinSCP.exe"))){
                            this.PathWinSCP = System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "WinSCP.exe");
                            return true;
                        }
                        else{
                            return false;
                        }
                    }
                    else {
                        return true;
                    }
                }
                else{
                    return false;
                }
            }
        }
        #endregion

        #region Constructeur
        public Parametrage()
        {

        }
        #endregion

        #region Methode Surcharger
        public override bool Equals(object obj)
        {
            Parametrage p = new Parametrage();
            p = (Parametrage)obj;
            if (this.Code.Equals(p.Code)) { return true; }
            else { return false; }
        }
        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append(this.Code);
            sb.Append(this.Libelle);
            sb.Append(this.Extention);
            return sb.ToString();
        }
        #endregion

    }
}
