﻿using System;
using System.IO;
using System.Collections.Generic;
using ExportCegidParam.BO;
using Microsoft.Office.Interop.Excel;

namespace Serialisation.EXCEL
{
    internal class SerialiseurExcel
    {
        #region "Gestion du singleton"
        private static SerialiseurExcel _instance = null;
        private SerialiseurExcel()
        {
        }

        public static SerialiseurExcel getInstance()
        {
            if (SerialiseurExcel._instance == null)
            {
                SerialiseurExcel._instance = new SerialiseurExcel();
            }
            return SerialiseurExcel._instance;
        }
        #endregion

        internal void Serialiser(string pathfile, DataExport _listData, Parametrage parametrage)
        {
            // on test dans cette méthode si les librairies excel (interop) sont installé sur la machine
            // si c'est le cas on utilise la méthode avec interop sinon sans
            //@"SOFTWARE\Microsoft\Office\Excel" // clé de registre d'excel

            if (ExportCegidParam.Outils.Registre.CheckAppInstalled(@"SOFTWARE\Microsoft\Office\Excel")){
                ExportWithInterop(pathfile, _listData, parametrage);
            }
            else{
                ExportWithoutInterop(pathfile,_listData,parametrage);
            }
        }


        private void ExportWithInterop(string pathFile, DataExport donnees, Parametrage parametrage)
        {
            Application xlApp = new Application();
            try
            {
                if (System.IO.File.Exists(pathFile)) { System.IO.File.Delete(pathFile); }
                Workbook book = xlApp.Workbooks.Add(Type.Missing);
                Worksheet ws = (Worksheet)book.ActiveSheet;
                ws.Name = parametrage.Libelle;
                int ligne = 1;
                //Ecriture des entetes de colonne
                if (parametrage.Entete)
                {
                    for (int i = 0; i < donnees.Entetes.Count; i++)
                    {
                        ws.Cells[(ligne), (i+1)] = donnees.Entetes[i];
                    }
                    ligne += 1;
                }
                //ecriture ligne à ligne (A SUPPRIMER)
                for (int i = 0; i < donnees.Lignes.Count; i++)
                {
                    for (int j = 0; j < (int)donnees.Lignes[i].Elements.Count; j++)
                    {
                        ws.Cells[(i + ligne), (j + 1)] = donnees.Lignes[i].Elements[j].Value.ToString();
                    }
                }
                book.SaveAs(pathFile, XlFileFormat.xlExcel7);
                book.Close();
            }
            catch (Exception ex)
            {
                throw new ApplicationException("Une Erreur est survenue lors de la génération du fichier Excel : " + ex.Message);
            }
            finally
            {
                xlApp.Quit();
            }
        }

        private void ExportWithoutInterop(string pathFile, DataExport listData, Parametrage parametrage)
        {
            try
            {
                if (File.Exists(pathFile)) { File.Delete(pathFile); }
                CarlosAg.ExcelXmlWriter.Workbook wb = new CarlosAg.ExcelXmlWriter.Workbook();
                CarlosAg.ExcelXmlWriter.Worksheet ws = wb.Worksheets.Add(parametrage.Libelle);
                CarlosAg.ExcelXmlWriter.WorksheetRow row;
                int ligne = 1;
                //Ecriture des entetes de colonne
                if (parametrage.Entete) {
                    row = ws.Table.Rows.Add();
                    row.Index = ligne;
                    for (int i = 0; i < listData.Entetes.Count; i++) { row.Cells.Add(listData.Entetes[i]); }
                    ligne += 1;
                }
                //ecriture ligne à ligne (A SUPPRIMER)
                for (int i = 0; i < listData.Lignes.Count; i++) {
                    row = ws.Table.Rows.Add();
                    row.Index = ligne + i;
                    for (int j = 0; j < (int)listData.Lignes[i].Elements.Count; j++) {
                        row.Cells.Add(listData.Lignes[i].Elements[j].Value.ToString());
                    }
                }
                wb.Save(pathFile);
            }
            catch (Exception ex)
            {
                throw new ApplicationException("Une Erreur est survenue lors de la génération du fichier Excel : " + ex.Message);
            }
        }

    }
}
